;+
; NAME:
;       iPolarPlot
;
; PURPOSE:
;       Produce a plot from data in polar coordinates.
;
; CATEGORY:
;       Graphics.
;
; CALLING SEQUENCE:
;       iPolarPlot, Theta, R
; INPUTS:
;
;       Theta = values of theta in radians.
;
;       R = values of radius.
;
; KEYWORD PARAMETERS:
;       Any of the keywords accepted by iPlot may be specified.
;       In addition to:
;
;   nrings =number of rings for the polar style overlay, only will appear on whole intervals
;   nspokes = number of spokes for polar overlay
;   ring_thick = thickness of rings
;   spoke_thick = thickness of spokes
;   labels = flag for placing angle and radius labels on overlay
;   rotate = rotation angle for the data to be ploted
;   charsize = size of characters on polar overlay
;   charthick = thickness of characters on polar overlay
;   overplot = plot new data on top of old plot, ROTATE MUST BE SET TO ORIGINAL VALUE
;   ringColor = color index for the rings, rgb triple
;   spokeColor = color index for the spokes, rgb triple
;   ringStyle = linestyle for for the rings
;   spokeStyle = linestyle for the spokes
;   spokeTicks = number of spoke tick marks on each spoke
;   ticklen = length of spoke ticks.
;   degrees = means that theta is in degrees
;
; OUTPUTS:
;       A polar plot is produced.
; COMMON BLOCKS:
;       None.
; SIDE EFFECTS:
;       None.
; RESTRICTIONS:
;       None.
; PROCEDURE:
;       The cartesian coordinates of each point are calculated.
; EXAMPLE:
;       nr = 20         ;# of radii
;       nt = 20         ;# of thetas
;       r = findgen(nr) / (nr-1)   ;Form r and Theta vectors
;       theta = 2 * !pi * findgen(nt) / (nt-1)
;       Create plots:
;       iPolarPlot,  theta, r, xstyle=5, ystyle=5, nspokes=9, nrings=5,spokeColor=[255,0,0], $
;                    ringColor=[0,255,0]
;
; Written by;
; Ronn Kling
; Kling Research and Software, inc.
; 7038 Westmoreland Dr.
; Warrenton, Va 20187
; ronn@rlkling.com
; www.rlkling.com
;
; MODIFICATION HISTORY:
;       Written by:
;       September, 2004 RLK
;-


pro iPolarPlot, t, r, $
        nrings=nrings, nspokes=nspokes, ring_thick=ring_thick, $
        spoke_thick=spoke_thick,labels=labels,rotate=rotate,$
        charsize=charsize,charthick=charthick, overplot=overplot, $
        ringColor=ringColor, spokeColor=spokeColor, degrees=degrees, $
        r_max=r_max, ringStyle=ringStyle, spokeStyle=spokeStyle, $
        spokeTicks=spokeTicks, ticklen=ticklen, labelColor=labelColor, $
  	    identifier=id, tool=tool, JservDefault=JservDefault, _extra=_extra

; Use a standard error trapping routine to handle
; any iTools system exceptions.  See the iTools
; Developer's Guide, pp 82, 83 for more info
on_error, 2
catch, iErr
if (iErr ne 0) then begin
       catch, /cancel
       IF obj_valid(oInitialData) THEN obj_destroy, oInitialData
       message, /reissue
       return
end

if n_params() ne 2 then message,'iPolarPlot must be called with r and theta data'

twopi = !dpi*2.d

if not keyword_set(ring_thick) then ring_thick=1
if not keyword_set(spoke_thick) then spoke_thick=1
if not keyword_set(rotate) then rotate = 0.0
if n_elements(spokeColor) eq 0 then spokeColor = !p.color
if n_elements(ringColor) eq 0 then ringColor = !p.color
if n_elements(labelColor) eq 0 then labelColor=!p.color
if keyword_set(degrees) then t = t *!dtor

;this next section is to override JSERV passed in values
if keyword_set(JservDefault) then begin
  nspokes= round(nspokes/4.0)*4 ;always a multiple of 4
endif

theta = t + (rotate*!dtor)
x = r * cos(theta)
y = r * sin(theta)
finalRing = 0.0 ;used for the spoke ticks

; Make a container object for the data objects,
; Then add them in
oInitialData = OBJ_NEW('IDLitParameterSet', NAME='Polar Plot Data')

; Create the data objects to hold the radial and angle vector
oDataRadial = obj_new('idlitDataIDLVector', x, NAME='RADIAL')
oDataTheta = obj_new('idlitDataIDLVector', y, NAME='THETA')
oInitialData->add, oDataRadial, PARAMETER_NAME='x'
oInitialData->add, oDataTheta, PARAMETER_NAME='y'

id=IDLitSys_CreateTool("Plot Tool",  $
        VISUALIZATION_TYPE=["PLOT"], $
        INITIAL_DATA=oInitialData, _EXTRA=_EXTRA, $
        style_name='noaxes', overplot=overplot, $
        TITLE="iPolarPlot Tool",/no_saveprompt)

;get the object reference
id = itGetCurrent(tool=oTool)

;if overplot then jump out
if keyword_set(overplot) then return

if keyword_set(nrings) then begin  ;rings done here
  ring = (findgen(360)/359.)*twopi
  if not keyword_set(r_max) then r_max = max(r)
  r_space = fix(r_max)/nrings ;side effect of only rings on whole number intverals
  if r_space le 0 then r_space=1
  if not keyword_set(r_max) then r_max = max([fix(r_max/r_space) * r_space,r_max])
  for i=1,r_max,r_space do begin
    oData = OBJ_NEW('IDLitParameterSet', NAME='Polar Plot Data')
    ringName = 'Ring_'  + strcompress(string(i),/remove)
    oRingX = obj_new('idlitDataIDLVector',i*cos(ring),name=ringName+'_X')
    oRingY = obj_new('idlitDataIDLVector',i*sin(ring),name=ringName+'_Y')

    oData->add, oRingX, PARAMETER_NAME='x'
    oData->add, oRingY, PARAMETER_NAME='y'

    if keyword_set(JservDefault) then begin
      ringStyle = (i mod 2) * 1
      if i eq r_max then ring_thick=ring_thick*2
    endif

    id=IDLitSys_CreateTool("Plot Tool",  $
        VISUALIZATION_TYPE=["PLOT"], $
        INITIAL_DATA=oData, $
        thick=ring_thick,color=ringColor,linestyle=ringStyle, $
        name = ringName, /overplot)
    finalRing = i
  endfor
endif

if keyword_set(nspokes) then begin  ;spokes done here
  theta_step = twopi/nspokes
  if not keyword_set(r_max) then r_max = max(r)
  for i=0.0,nspokes-1 do begin
    index = i * theta_step
    oData = OBJ_NEW('IDLitParameterSet', NAME='Polar Plot Data')
    spokeName = 'Spoke_'  + strcompress(string(index/!dtor),/remove)
    oSpokeX = obj_new('idlitDataIDLVector',[0,r_max*cos(index)],name=spokeName+'_X')
    oSpokeY = obj_new('idlitDataIDLVector',[0,r_max*sin(index)],name=spokeName+'_Y')

    oData->add, oSpokeX, PARAMETER_NAME='x'
    oData->add, oSpokeY, PARAMETER_NAME='y'

    if keyword_set(JservDefault) then begin
      case (i/nspokes) of
        0.0 : spoke_thick=2
        0.25: spoke_thick=2
        0.50: spoke_thick=2
        0.75: spoke_thick=2
        else : spoke_thick=1
      endcase
    endif

    id=IDLitSys_CreateTool("Plot Tool",  $
        VISUALIZATION_TYPE=["PLOT"], $
        INITIAL_DATA=oData, $
        thick=spoke_thick, color=spokeColor,linestyle=spokeStyle, $
        name = spokeName, /overplot)
  endfor
endif

if keyword_set(labels) then begin
  ;have to add labels to the visualization layer, not the annotation layer.
  ;if they are in the annotation layer they will not move with the plot
  temp = oTool->findIdentifiers('*VISUALIZATION LAYER')
  ;vis layer is always(?) then second one?
  idVisLayer = temp[1]
  oVisLayer = oTool->getByIdentifier(idVisLayer)
  ;add spoke labels
  for i=0.0,nspokes-1,keyword_set(JservDefault)? (nspokes/4): 1 do begin
    angleForText = (i * theta_step) - (rotate * !dtor)
    angle = (i * theta_step) + 0.0001 ;fudge factor for the angles
    xPos = r_max*cos(angle) & yPos = r_max*sin(angle)
    textval = ((float(angleForText)/!dtor)) mod 360.
    textval = strcompress(string(textval,format='(f5.1)'))
    if (keyword_set(JservDefault) and (textval eq 0.0)) then textVal = textVal + ' RAAN'
    ;placement of alignment depends upon where on the polar plot they are
    ;this is defined like a standard polar plot, i.e. and angle of 0
    ;is at the far right
    if ( (angle ge 0.0) and (angle lt !pi/2.)) then begin
      align = 0.0
      vertAlign = 0.5
      xPos = xPos + r_space/10.0
    endif else if ( (angle ge !pi/2.) and (angle lt !pi)) then begin
      align = 0.5
      vertAlign = 0.0
      yPos = yPos + r_space/10.0
    endif else if ( (angle ge !pi) and (angle lt !pi*1.5)) then begin
      align = 1.0
      vertAlign = 0.5
      xPos = xPos - r_space/10.0
    endif else if ( (angle ge !pi*1.5) and (angle le !pi*2.0)) then begin
      align = 0.5
      vertAlign = 1.0
      yPos = yPos - r_space/10.0
    endif
    ;create the iTools text object for a label
    oSpokeText = obj_new('IDLitVisText', _string=textval,location=[xPos,yPos,0], $
                 align=align, VERTICAL_ALIGNMENT=vertAlign);, $
                 ;font=obj_new('IDLgrFont','arial'))
    oVisLayer->add, oSpokeText
  endfor
  if keyword_set(JservDefault) then begin
    lastR = r_max - 2
  endif else begin
    lastR = r_max
  endelse

  for i=(1+keyword_set(JservDefault)),lastR,(r_space * (keyword_set(JservDefault)? 2: 1)) do begin
    if keyword_set(JservDefault) then rotate=0
    xPos = i*cos(rotate*!dtor) & yPos = i*sin(rotate*!dtor)
    textVal = strcompress(string(i,format='(i3)'))
    oSpokeText = obj_new('IDLitVisText', _string=textval,location=[xPos,yPos,0], $
                 alignment=1.0)
    oVisLayer->add, oSpokeText
  endfor
endif

return & end
